// SequenceType.h: interface for the SequenceType class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _SEQUENCETYPE_HPP
#define _SEQUENCETYPE_HPP

#include <pathan/Pathan.hpp>

#include <pathan/Sequence.hpp>
#include <pathan/internal/parser/QName.hpp>

class Item;
class XPath2MemoryManager;

class PATHAN_EXPORT SchemaType
{
public:
    SchemaType();
    SchemaType(QualifiedName* globalType);
    ~SchemaType();

    bool _rootIsType;
    QualifiedName* _root;
    VectorOfQualifiedNames* _steps;
};

class PATHAN_EXPORT SequenceType  
{
public:

  class PATHAN_EXPORT ItemType
  {
  public:

    /**
     * The type of item that this sequence can hold.
     * TEST_ELEMENT      - element node
     * TEST_ATTRIBUTE    - attribute node
     * TEST_NODE         - node
     * TEST_PI           - processing instruction node
     * TEST_COMMENT      - comment node
     * TEST_TEXT         - text node
     * TEST_DOCUMENT     - document node
     * TEST_ANYTHING     - any item
     * TEST_ATOMIC_TYPE  - the named atomic type or parent's named atomic type if derived through restriction
     */
    typedef enum
    {
      TEST_ELEMENT,
      TEST_ATTRIBUTE,
      TEST_NODE,
      TEST_PI,
      TEST_COMMENT,
      TEST_TEXT,
      TEST_DOCUMENT,
      TEST_ANYTHING,
      TEST_ATOMIC_TYPE
    } ItemTestType;

    ItemType(ItemTestType test, QualifiedName* name=NULL, QualifiedName* type=NULL);
    ItemType(ItemTestType test, QualifiedName* name, SchemaType* type);

    /**
     * Destructor.
     */
    ~ItemType();

    /**
     * Getter for m_nTestType
     */
    ItemTestType getItemTestType(void) const;

    void setAllowNilled(bool value);
    bool getAllowNilled() const;

    QualifiedName *getName(void) const;
    QualifiedName *getType(void) const;

    void setName(QualifiedName * name);
    void setType(QualifiedName * type);

    void setNameURI(const XMLCh* const nameURI);
    void setTypeURI(const XMLCh* const typeURI);

    //Return a URI, firstly the overriding URI string, if not, the URI bond to the QName prefix
    const XMLCh* getTypeURI(DynamicContext* context) const;
    const XMLCh* getNameURI(DynamicContext* context) const;

    bool matches(const Item* toBeTested, DynamicContext* context) const;
    bool matches(const XERCES_CPP_NAMESPACE_QUALIFIER DOMNode* toBeTested, DynamicContext* context) const;
    bool matchesNameType(const Item* toBeTested, DynamicContext* context) const;

  protected:

    // The ItemTestType of this ItemType
    ItemTestType m_nTestType;

    // The name and type to match
    QualifiedName* m_pName, * m_pType;
    SchemaType* m_pSchemaType;

    // The forced URIs for name and type
    const XMLCh* m_NameURI, * m_TypeURI;

    // allow elements having the xsi:nil="true" attribute
    bool m_bAllowNil;
  };//ItemType



  /**
   * Number of occurances of the ItemType.
   * STAR specifies zero or more occurances.
   * PLUS specifies one or more occurances.
   * QUESTION_MARK specifies zero or one occurance.
   */
  typedef enum 
  {
    EXACTLY_ONE=0,
    STAR=1,
    PLUS=2,
    QUESTION_MARK=3
  } OccurrenceIndicator;

  /**
   * Constructor for atomic types
   */
  SequenceType(const XMLCh* typeURI,const XMLCh* typeName, OccurrenceIndicator occur = EXACTLY_ONE);

  /**
   * Generic constructor.
   */
  SequenceType(ItemType* test, OccurrenceIndicator occur = EXACTLY_ONE);

  /**
   * Default constructor.
   */
  SequenceType();

  /**
   * Destructor.
   */
  virtual ~SequenceType();

  /** 
   * Setter for m_pItemType.
   */ 
  void setItemType(ItemType* itemType);

  /**
   * Setter for m_nOccurrencer.
   */
  void setOccurrence(OccurrenceIndicator nOccurrence);

  /**
   * Setter for the nameURI
   */
  void setNameURI(const XMLCh* const nameURI);

  /**
   * Setter for the typeURI
   */
  void setTypeURI(const XMLCh* const typeURI);

  //Return a URI, firstly the overriding URI string, if not, the URI bond to the QName prefix
  const XMLCh* getTypeURI(DynamicContext* context) const;
  const XMLCh* getNameURI(DynamicContext* context) const;

  /**
   * Returns true if the given sequence matches ItemType for every item 
   * in the given Sequence and the number of such items.
   * Returns false otherwise.
   */
  bool matches(const Sequence& toBeTested, DynamicContext* context) const;

  Sequence castAsFunction(const Sequence &param, DynamicContext* context) const;

  QualifiedName *getConstrainingType(void) const;
  QualifiedName *getConstrainingName(void) const;

  /**
   * Getter for m_pItemType.
   */
  ItemType::ItemTestType getItemTestType(void) const;

  /**
   * Getter for m_nOccurrence.
   */
  OccurrenceIndicator getOccurrenceIndicator(void) const;

  const ItemType *getItemType() const;

protected:

  // The ItemType of this SequenceType
  ItemType* m_pItemType;
  
  // The OccuranceIndicator of this SequenceType
  OccurrenceIndicator m_nOccurrence;
};

#endif // _SEQUENCETYPE_HPP
